/*
  XVarStar, GCVS catalogue explorer
  Copyright (C) 2001-2008 Ivan Vitjuk 

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <string.h>
#include <stdarg.h>

static int var_error;
static char var_exterror[256];

struct var_error_strings{
        int code;
        char *str;
};

static struct var_error_strings var_errors[] = {
        {0, "No error"},
        {1, "Invalid argument value"},
        {2, "Can't load GCVS catalogue"},
        {3, "Invalid Ra coordinates"},
        {4, "Invalid Dec coordinates"},
        {5, "Unable to configure"},
        {6, "Can't open output file"},
        {7, "Unknown search field"},
        {8, "Unknown constellation"},
        {9, "No stars found"},
        {10, "Unable to determine location of configuration file"},
        {11, "No search criteria slected"}
};

static const size_t VAR_ERROR_STRINGS_SIZE = sizeof(var_errors)/sizeof(*var_errors);

int var_is_error();
void var_set_error(int err);
void var_error_reason(char *fmt,...);
void var_perror(char *msg, FILE *fp);
char *var_strerror(char *msg, char *buf, size_t buflen);

static char *var_errorstr(int error)
{
        register size_t count = VAR_ERROR_STRINGS_SIZE;

        do{
                count--;
        }while((count >= 0) && (error != var_errors[count].code));

        if(count < 0)
                return NULL;
        
        return var_errors[count].str;
}

int var_get_error()
{
        return var_error;
}

void var_set_error(int err)
{
        var_error = err;
}

void var_error_reason(char *fmt,...)
{
	va_list args;

        if(!fmt){
                memset(var_exterror, 0, sizeof(var_exterror));
                return;
        }

        var_exterror[0] = 0;
        va_start(args, fmt);
        vsnprintf(var_exterror, sizeof(var_exterror) - 1, fmt, args);
        va_end(args);
}

void var_perror(char *msg, FILE *fp)
{
        if(!fp)
                fp = stderr;

        fprintf(fp, "[VARIABLE error] ");
        if(var_exterror[0])
                fprintf(fp, "%s(): %s [%s]\n", msg, 
                        var_errorstr(var_error), var_exterror);
        else
                fprintf(fp, "%s(): %s\n", msg, 
                        var_errorstr(var_error));

        var_set_error(0);
        var_error_reason(NULL);
}

char *var_strerror(char *msg, char *buf, size_t buflen)
{
        static char my_buf[256], *tmp;
        size_t tmplen;

        if(buf){
                tmp = buf;
                tmplen = buflen;
        }else{
                tmp = my_buf;
                tmplen = sizeof(my_buf);
        }
        
        if(var_exterror[0])
                snprintf(tmp, tmplen, "%s(): %s [%s]", msg, 
                         var_errorstr(var_error), var_exterror);
        else
                snprintf(tmp, tmplen, "%s(): %s", msg, 
                         var_errorstr(var_error));

        tmp[tmplen-1] = '\0';
        var_set_error(0);
        var_error_reason(NULL);
        return tmp;
}
